(use gauche.uvector)

(use sdl)
(use gl)


(define color0 (list->f32vector '(1.0 1.0 0.0)))
(define color1 (list->f32vector '(1.0 0.0 0.0)))
(define color2 (list->f32vector '(0.0 0.0 0.0)))
(define color3 (list->f32vector '(0.0 1.0 0.0)))
(define color4 (list->f32vector '(0.0 1.0 1.0)))
(define color5 (list->f32vector '(1.0 1.0 1.0)))
(define color6 (list->f32vector '(1.0 0.0 1.0)))
(define color7 (list->f32vector '(0.0 0.0 1.0)))

(define cube0 (list->f32vector '( 0.5  0.5 -0.5)))
(define cube1 (list->f32vector '( 0.5 -0.5 -0.5)))
(define cube2 (list->f32vector '(-0.5 -0.5 -0.5)))
(define cube3 (list->f32vector '(-0.5  0.5 -0.5)))
(define cube4 (list->f32vector '(-0.5  0.5  0.5)))
(define cube5 (list->f32vector '( 0.5  0.5  0.5)))
(define cube6 (list->f32vector '( 0.5 -0.5  0.5)))
(define cube7 (list->f32vector '(-0.5 -0.5  0.5)))


;;
;; main
;;
(define (main args)
  (sdl-init SDL_INIT_VIDEO)

  (sdl-gl-set-attribute SDL_GL_RED_SIZE 5)
  (sdl-gl-set-attribute SDL_GL_GREEN_SIZE 5)
  (sdl-gl-set-attribute SDL_GL_BLUE_SIZE 5)
  (sdl-gl-set-attribute SDL_GL_DEPTH_SIZE 16)
  (sdl-gl-set-attribute SDL_GL_DOUBLEBUFFER 1)

  (let ((screen (sdl-set-video-mode 640 480 16 SDL_OPENGL)))
    (cond ((not (sdl-surface? screen))
           (format #t "ERROR: ~A\n" (sdl-get-error))
           (sdl-quit)
           (exit)))

    (sdl-wm-set-caption "SDL GL test" "testgl")

    (format #t "Vendor:     ~a\n" (gl-get-string GL_VENDOR))
    (format #t "Renderer:   ~a\n" (gl-get-string GL_RENDERER))
    (format #t "Version:    ~a\n" (gl-get-string GL_VERSION))
    (format #t "Extensions: ~a\n" (gl-get-string GL_EXTENSIONS))


    (gl-viewport 0 0 640 480)

    (gl-matrix-mode GL_PROJECTION)
    (gl-load-identity)

    (gl-ortho -2.0 2.0 -2.0 2.0 -20.0 20.0)

    (gl-matrix-mode GL_MODELVIEW)
    (gl-load-identity)

    (gl-enable GL_DEPTH_TEST)

    (gl-depth-func GL_LESS)

    (gl-shade-model GL_SMOOTH)

    (let ((start-time (sdl-get-ticks))
          (frames 0))
      (let loop ((e (sdl-make-event)))
        (gl-clear-color 0.0 0.0 0.0 1.0)
        (gl-clear (logior GL_COLOR_BUFFER_BIT GL_DEPTH_BUFFER_BIT))

        (gl-begin GL_QUADS)

        (gl-color  color0)
        (gl-vertex cube0)
        (gl-color  color1)
        (gl-vertex cube1)
        (gl-color  color2)
        (gl-vertex cube2)
        (gl-color  color3)
        (gl-vertex cube3)

        (gl-color  color3)
        (gl-vertex cube3)
        (gl-color  color4)
        (gl-vertex cube4)
        (gl-color  color7)
        (gl-vertex cube7)
        (gl-color  color2)
        (gl-vertex cube2)

        (gl-color  color0)
        (gl-vertex cube0)
        (gl-color  color5)
        (gl-vertex cube5)
        (gl-color  color6)
        (gl-vertex cube6)
        (gl-color  color1)
        (gl-vertex cube1)

        (gl-color  color5)
        (gl-vertex cube5)
        (gl-color  color4)
        (gl-vertex cube4)
        (gl-color  color7)
        (gl-vertex cube7)
        (gl-color  color6)
        (gl-vertex cube6)

        (gl-color  color5)
        (gl-vertex cube5)
        (gl-color  color0)
        (gl-vertex cube0)
        (gl-color  color3)
        (gl-vertex cube3)
        (gl-color  color4)
        (gl-vertex cube4)

        (gl-color  color6)
        (gl-vertex cube6)
        (gl-color  color1)
        (gl-vertex cube1)
        (gl-color  color2)
        (gl-vertex cube2)
        (gl-color  color7)
        (gl-vertex cube7)

        (gl-end)

        (gl-matrix-mode GL_MODELVIEW)
        (gl-rotate 5.0 1.0 1.0 1.0)

        (sdl-gl-swap-buffers)

        (inc! frames)

        (cond ((and (> (sdl-poll-event e) 0)
                    (= SDL_KEYUP (sdl-event-type e))
                    (= SDLK_ESCAPE (sdl-event-key-keysym-sym e)))
               #t)
              (else
               (sdl-delay 10)
               (loop e))))

      (let ((this-time (sdl-get-ticks)))
        (if (not (= this-time start-time))
            (format #t "FPS: ~d\n" (* (/ frames (- this-time start-time)) 1000))))))

  (sdl-quit))
